/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.sun.mojarra.scales.component;

import com.sun.mojarra.scales.util.CssUtil;
import com.sun.mojarra.scales.util.RenderingHelper;
import com.sun.mojarra.scales.util.ScalesUtil;
import com.sun.mojarra.scales.util.YuiConstants;
import java.io.IOException;
import java.util.Map;
import javax.faces.component.UIComponent;
import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 *
 * @author jasonlee
 */
public class Carousel extends UIOutput {
    public static final String COMPONENT_FAMILY = "com.sun.mojarra.scales.Carousel";
    public static final String COMPONENT_TYPE = COMPONENT_FAMILY;
    public static final String RENDERER_TYPE = COMPONENT_FAMILY;
    private Object[] _state = null;
    protected String animation = "{ speed: 0, effect: null }";
    protected Integer autoPlay = 0;
    protected Integer firstVisible = 0;
    protected Boolean isCircular = Boolean.FALSE;
    protected Boolean isVertical = Boolean.FALSE;
    protected String navigation;
    protected Integer numVisible = 3;
    protected Double revealAmount = 0d;
    protected Integer scrollIncrement = 1;
    protected String width;
    protected String style = "";
    protected String styleClass = "";

    public Carousel() {
        super();
        setRendererType(RENDERER_TYPE);

        FacesContext context = FacesContext.getCurrentInstance();
        if (context != null) {
            String baseUrl = context.getExternalContext().getRequestContextPath();
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_CAROUSEL));
            CssUtil.linkCarouselOverrides(context);
        }
    }

    @Override
    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    @Override
    public void encodeBegin(FacesContext context) throws IOException {
        super.encodeBegin(context);

        // suppress rendering if "rendered" property on the component is false.
        if (!isRendered()) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        Map<String, Object> attrs = getAttributes();
        writer.startElement("div", this);
        writer.writeAttribute("id", getClientId(context), "id");
        writer.writeAttribute("class", attrs.get("styleClass"), "class");
        writer.writeAttribute("style", attrs.get("style"), "style");
        writer.startElement("ol", this);
    }

    @Override
    public void encodeChildren(FacesContext context) throws IOException {
        ResponseWriter writer = context.getResponseWriter();
        for (UIComponent child : getChildren()) {
            writer.startElement("li", this);
            child.encodeAll(context);
            writer.endElement("li");
        }
    }

    @Override
    public void encodeEnd(FacesContext context) throws IOException {
        super.encodeEnd(context);
        if (!isRendered()) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        writer.endElement("ol");
        writer.endElement("div");
        RenderingHelper.writeOnDomReady(writer, "scales.createCarousel('" + getClientId(context) + "', {" + buildConfig(context) + "});");
    }

    @Override
    public boolean getRendersChildren() {
        return true;
    }

    public String getAnimation() {
        return animation;
    }

    public void setAnimation(String animation) {
        this.animation = animation;
    }

    public Integer getAutoPlay() {
        return autoPlay;
    }

    public void setAutoPlay(Integer autoPlay) {
        this.autoPlay = autoPlay;
    }

    public Integer getFirstVisible() {
        return firstVisible;
    }

    public void setFirstVisible(Integer firstVisible) {
        this.firstVisible = firstVisible;
    }

    public Boolean getIsCircular() {
        return isCircular;
    }

    public void setIsCircular(Boolean isCircular) {
        this.isCircular = isCircular;
    }

    public Boolean getIsVertical() {
        return isVertical;
    }

    public void setIsVertical(Boolean isVertical) {
        this.isVertical = isVertical;
    }

    public String getNavigation() {
        return navigation;
    }

    public void setNavigation(String navigation) {
        this.navigation = navigation;
    }

    public Integer getNumVisible() {
        return numVisible;
    }

    public void setNumVisible(Integer numVisible) {
        this.numVisible = numVisible;
    }

    public Double getRevealAmount() {
        return revealAmount;
    }

    public void setRevealAmount(Double revealAmount) {
        this.revealAmount = revealAmount;
    }

    public Integer getScrollIncrement() {
        return scrollIncrement;
    }

    public void setScrollIncrement(Integer scrollIncrement) {
        this.scrollIncrement = scrollIncrement;
    }

    public String getStyle() {
        return style;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        return styleClass;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    public String getWidth() {
        return width;
    }

    public void setWidth(String width) {
        this.width = width;
    }

    protected String buildConfig(FacesContext context) {
        StringBuilder builder = new StringBuilder();
        Map<String, Object> attrs = getAttributes();
        ScalesUtil.appendToConfig(builder, "isCircular", attrs.get("isCircular"), false, false);
        ScalesUtil.appendToConfigIfNotNull(builder, "numVisible", attrs.get("inumVisible"), false, true);
        ScalesUtil.appendToConfig(builder, "animation", attrs.get("animation"), false, true);
        ScalesUtil.appendToConfig(builder, "autoPlay", attrs.get("autoPlay"), false, true);
        ScalesUtil.appendToConfig(builder, "firstVisible", attrs.get("firstVisible"), false, true);
        ScalesUtil.appendToConfig(builder, "isVertical", attrs.get("isVertical"), false, true);
        ScalesUtil.appendToConfigIfNotNull(builder, "navigation", attrs.get("navigation"), false, true);
        ScalesUtil.appendToConfig(builder, "revealAmount", attrs.get("revealAmount"), false, true);
        ScalesUtil.appendToConfig(builder, "scrollIncrement", attrs.get("scrollIncrement"), false, true);
        ScalesUtil.appendToConfigIfNotNull(builder, "width", attrs.get("width"), true, true);

        return builder.toString();
    }

    @Override
    public void restoreState(FacesContext _context, Object _state) {
        this._state = (Object[]) _state;
        super.restoreState(_context, this._state[0]);
        animation = (String) this._state[1];
        autoPlay = (Integer) this._state[2];
        firstVisible = (Integer) this._state[3];
        isCircular = (Boolean) this._state[4];
        isVertical = (Boolean) this._state[5];
        navigation = (String) this._state[6];
        numVisible = (Integer) this._state[7];
        revealAmount = (Double) this._state[8];
        scrollIncrement = (Integer) this._state[9];
        width = (String) this._state[10];
        style = (String) this._state[11];
        styleClass = (String) this._state[12];
    }

    @Override
    public Object saveState(FacesContext _context) {
        if (_state == null) {
            _state = new Object[13];
        }
        _state[0] = super.saveState(_context);
        _state[1] = animation;
        _state[2] = autoPlay;
        _state[3] = firstVisible;
        _state[4] = isCircular;
        _state[5] = isVertical;
        _state[6] = navigation;
        _state[7] = numVisible;
        _state[8] = revealAmount;
        _state[9] = scrollIncrement;
        _state[10] = width;
        _state[11] = style;
        _state[12] = styleClass;

        return _state;
    }
}