package com.sun.mojarra.scales.component;

import com.sun.mojarra.scales.util.CssUtil;
import com.sun.mojarra.scales.util.RenderingHelper;
import com.sun.mojarra.scales.util.ScalesUtil;
import com.sun.mojarra.scales.util.YuiConstants;
import java.io.IOException;
import java.util.Map;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 *
 * @author Jason Lee
 */
public class ColorPicker extends UIInput {

    public static final String COMPONENT_FAMILY = "com.sun.mojarra.scales.component.ColorPicker";
    public static final String RENDERER_TYPE = COMPONENT_FAMILY;
    public static final String COMPONENT_TYPE = COMPONENT_FAMILY;

    // Component Properties
    private boolean showControls = true;
    private boolean showHexControls = true;
    private boolean showHexSummary = true;
    private boolean showHsvControls = false;
    private boolean showRgbControls = true;
    private boolean showWebSafe = true;
    private String backgroundColor = "#eeeeee";
    private String style = "";
    private String styleClass = "";
    // Internal variables
    private Object[] _state = null;

    public ColorPicker() {
        super();
        setRendererType(RENDERER_TYPE);

        FacesContext context = FacesContext.getCurrentInstance();
        if (context != null) {
            String baseUrl = context.getExternalContext().getRequestContextPath();
            Links.addCss(Links.EXTERNAL_STYLESHEETS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.CSS_COLOR_PICKER));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_SLIDER));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_COLORPICKER));
            CssUtil.linkColorPickerOverrides(context);

//            setValue("000000");
        }
    }

    @Override
    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    @Override
    public Object saveState(FacesContext context) {
        if (_state == null) {
            _state = new Object[10];
        }
        _state[0] = super.saveState(context);
        _state[1] = showControls;
        _state[2] = showHexControls;
        _state[3] = showHexSummary;
        _state[4] = showHsvControls;
        _state[5] = showRgbControls;
        _state[6] = showWebSafe;
        _state[7] = backgroundColor;
        _state[8] = style;
        _state[9] = styleClass;
        return _state;
    }

    @Override
    public void restoreState(FacesContext context, Object state) {
        this._state = (Object[]) state;
        super.restoreState(context, this._state[0]);
        showControls = (Boolean) _state[1];
        showHexControls = (Boolean) _state[2];
        showHexSummary = (Boolean) _state[3];
        showHsvControls = (Boolean) _state[4];
        showRgbControls = (Boolean) _state[5];
        showWebSafe = (Boolean) _state[6];
        backgroundColor = (String) _state[7];
        style = (String) _state[8];
        styleClass = (String) _state[9];
    }

    @Override
    public void encodeBegin(FacesContext context) throws IOException {
        super.encodeBegin(context);

        // suppress rendering if "rendered" property on the component is false.
        if (!isRendered()) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        String clientId = getClientId(context);
        Map<String, Object> attrs = getAttributes();

        writer.startElement("span", this);
        writer.writeAttribute("class", attrs.get("styleClass"), "class");
        writer.writeAttribute("style", attrs.get("style"), "style");

        RenderingHelper.outputElement(writer, this, "div",
                                      new String[][]{{"id", clientId}, {"class", (String)attrs.get("styleClass")},
                                                     {"style", "position: relative; padding: 6px; background-color: #" + 
                                                              (String)attrs.get("backgroundColor") + "; width: 420px; height:220px; " +
                                                              (String)attrs.get("style") + "}"}});
        RenderingHelper.writeOnDomReady(writer, "scales.createColorPicker('" + clientId + "', {" + buildConfig(context) + "});");
    }

    public String getBackgroundColor() {
        return backgroundColor;
    }

    public void setBackgroundColor(String backgroundColor) {
        this.backgroundColor = backgroundColor;
    }

    public boolean isShowControls() {
        return showControls;
    }

    public void setShowControls(boolean showControls) {
        this.showControls = showControls;
    }

    public boolean isShowHexControls() {
        return showHexControls;
    }

    public void setShowHexControls(boolean showHexControls) {
        this.showHexControls = showHexControls;
    }

    public boolean isShowHexSummary() {
        return showHexSummary;
    }

    public void setShowHexSummary(boolean showHexSummary) {
        this.showHexSummary = showHexSummary;
    }

    public boolean isShowHsvControls() {
        return showHsvControls;
    }

    public void setShowHsvControls(boolean showHsvControls) {
        this.showHsvControls = showHsvControls;
    }

    public boolean isShowRgbControls() {
        return showRgbControls;
    }

    public void setShowRgbControls(boolean showRgbControls) {
        this.showRgbControls = showRgbControls;
    }

    public boolean isShowWebSafe() {
        return showWebSafe;
    }

    public void setShowWebSafe(boolean showWebSafe) {
        this.showWebSafe = showWebSafe;
    }

    public String getStyle() {
        return style;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        return styleClass;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    protected String buildConfig(FacesContext context) {
        StringBuilder builder = new StringBuilder();
        Map<String, Object> attrs = getAttributes();
        String baseUrl = ScalesUtil.getStaticResourceUrl(context, "");
        String value = (String) attrs.get("value");

        
        ScalesUtil.appendToConfig(builder, "showcontrols", (Boolean) attrs.get("showControls"), false, false);
        ScalesUtil.appendToConfig(builder, "showhexcontrols", (Boolean) attrs.get("showHexControls"), false, true);
        ScalesUtil.appendToConfig(builder, "showhexsummary", (Boolean) attrs.get("showHexSummary"), false, true);
        ScalesUtil.appendToConfig(builder, "showhsvcontrols", (Boolean) attrs.get("showHsvControls"), false, true);
        ScalesUtil.appendToConfig(builder, "showrgbcontrols", (Boolean) attrs.get("showRgbControls"), false, true);
        ScalesUtil.appendToConfig(builder, "showwebsafe", (Boolean) attrs.get("showWebSafe"), false, true);
        ScalesUtil.appendToConfig(builder, "images", "{PICKER_THUMB: '" + baseUrl + "yui/colorpicker/assets/picker_thumb.png',HUE_THUMB: '" +
                baseUrl + "yui/colorpicker/assets/hue_thumb.png'}", false, true);

        if (value != null) {
            addRgbToConfig(builder, value);
        }

        return builder.toString();
    }

    @Override
    // TODO: <!decode   convertDate(dateFormat="#{param.$this{clientId}_dateFormat}") />
    public void decode(FacesContext facesContext) {
        Map requestMap = facesContext.getExternalContext().getRequestParameterMap();
        setSubmittedValue((String) requestMap.get("yui-picker-hex"));
    }

    protected void addRgbToConfig(StringBuilder builder, String color) {
        String red;
        String green;
        String blue;

        if (color == null) {
            return;
        }
        if (color.startsWith("#")) {
            color = color.substring(1);
        }
        if (color.length() == 3) {
            red = color.substring(0, 1) + color.substring(0, 1);
            green = color.substring(1, 2) + color.substring(1, 2);
            blue = color.substring(2, 3) + color.substring(2, 3);
        } else {
            red = color.substring(0, 2);
            green = color.substring(2, 4);
            blue = color.substring(4, 6);
        }
        ScalesUtil.appendToConfig(builder, "red", Integer.parseInt(red, 16), false, true);
        ScalesUtil.appendToConfig(builder, "green", Integer.parseInt(green, 16), false, true);
        ScalesUtil.appendToConfig(builder, "blue", Integer.parseInt(blue, 16), false, true);
    }
}
