/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.mojarra.scales.component;

import com.sun.mojarra.scales.util.CssUtil;
import com.sun.mojarra.scales.util.RenderingHelper;
import com.sun.mojarra.scales.util.ScalesUtil;
import com.sun.mojarra.scales.util.YuiConstants;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.logging.Logger;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * @author Jason Lee
 */
public class DateSelector extends UIInput {
    public static final String COMPONENT_FAMILY = "com.sun.mojarra.scales.DateSelector";
    public static final String COMPONENT_TYPE = DateSelector.COMPONENT_FAMILY;
    public static final String RENDERER_TYPE = DateSelector.COMPONENT_FAMILY;
    private Object[] _state = null;
    protected Boolean hideBlankWeeks = Boolean.FALSE;
    protected String locale = "en_US";
    protected Boolean multiSelect = Boolean.FALSE;
    protected String navigator = "true";
    protected String onChange = "";
    protected Boolean popup = Boolean.TRUE;
    protected Boolean showWeekdays = Boolean.TRUE;
    protected Boolean showWeekFooter = Boolean.FALSE;
    protected Boolean showWeekHeader = Boolean.FALSE;
    protected Integer startWeekday = 0;
    protected Date minDate;
    protected Date maxDate;
    protected Boolean showMenus;
    protected String format = "yyyy/MM/dd";
    protected Boolean enableManualInput = Boolean.FALSE;
    protected String style;
    protected String styleClass;

    public DateSelector() {
        super();
        setRendererType(RENDERER_TYPE);

        FacesContext context = FacesContext.getCurrentInstance();
        if (context != null) {
            String baseUrl = context.getExternalContext().getRequestContextPath();
            Links.addCss(Links.EXTERNAL_STYLESHEETS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.CSS_SKIN_SAM));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_CALENDAR));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_DATESELECTOR_HELPER));
            CssUtil.linkDateSelectorOverrides(context);
        }
    }

    @Override
    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    @Override
    public void encodeBegin(FacesContext context) throws IOException {
        super.encodeBegin(context);

        // suppress rendering if "rendered" property on the component is false.
        if (!isRendered()) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        Map<String, Object> attrs = getAttributes();
        String formattedDate = ScalesUtil.formatDate((Date) getValue(), (String)attrs.get("format"), false);
        String clientId = getClientId(context);
        String baseUrl = ScalesUtil.getStaticResourceUrl(context, "");

        writer.startElement("span", this);
        writer.writeAttribute("class", (String)attrs.get("styleClass"), "class");
        writer.writeAttribute("style", (String)attrs.get("style"), "style");

        RenderingHelper.outputElement(writer, this, "input",
                                      new String[][]{{"id", clientId}, {"name", clientId}, {"type", "text"}, {"size", "10"},
                                                     {"value", formattedDate}, {"onchange", (String)attrs.get("onChange")}});
        RenderingHelper.outputElement(writer, this, "span", new String[][]{{"id", clientId + "Container"}});
        RenderingHelper.outputElement(writer, this, "img",
                                      new String[][]{{"id", clientId + "Trigger"}, {"alt", "calendar"}, {"style", "vertical-align: middle"},
                                                     {"src", baseUrl + "scales/dateselector_icon.gif"}});
        writer.endElement("span");
        RenderingHelper.writeOnDomReady(writer, "scales.createDateSelector('" + clientId + "', " + buildConfig(context) + ");");
    }

    public Boolean getEnableManualInput() {
        return enableManualInput;
    }

    public void setEnableManualInput(Boolean enableManualInput) {
        this.enableManualInput = enableManualInput;
    }

    public String getFormat() {
        return format;
    }

    public void setFormat(String format) {
        this.format = format;
    }

    public Boolean getHideBlankWeeks() {
        return hideBlankWeeks;
    }

    public void setHideBlankWeeks(Boolean hideBlankWeeks) {
        this.hideBlankWeeks = hideBlankWeeks;
    }

    public String getLocale() {
        return locale;
    }

    public void setLocale(String locale) {
        this.locale = locale;
    }

    public Date getMaxDate() {
        return maxDate;
    }

    public void setMaxDate(Date maxDate) {
        this.maxDate = maxDate;
    }

    public Date getMinDate() {
        return minDate;
    }

    public void setMinDate(Date minDate) {
        this.minDate = minDate;
    }

    public Boolean getMultiSelect() {
        return multiSelect;
    }

    public void setMultiSelect(Boolean multiSelect) {
        this.multiSelect = multiSelect;
    }

    public String getNavigator() {
        return navigator;
    }

    public void setNavigator(String navigator) {
        this.navigator = navigator;
    }

    public String getOnChange() {
        return onChange;
    }

    public void setOnChange(String onChange) {
        this.onChange = onChange;
    }

    public Boolean getPopup() {
        return popup;
    }

    public void setPopup(Boolean popup) {
        this.popup = popup;
    }

    public Boolean getShowMenus() {
        return showMenus;
    }

    public void setShowMenus(Boolean showMenus) {
        this.showMenus = showMenus;
    }

    public Boolean getShowWeekFooter() {
        return showWeekFooter;
    }

    public void setShowWeekFooter(Boolean showWeekFooter) {
        this.showWeekFooter = showWeekFooter;
    }

    public Boolean getShowWeekHeader() {
        return showWeekHeader;
    }

    public void setShowWeekHeader(Boolean showWeekHeader) {
        this.showWeekHeader = showWeekHeader;
    }

    public Boolean getShowWeekdays() {
        return showWeekdays;
    }

    public void setShowWeekdays(Boolean showWeekdays) {
        this.showWeekdays = showWeekdays;
    }

    public Integer getStartWeekday() {
        return startWeekday;
    }

    public void setStartWeekday(Integer startWeekday) {
        this.startWeekday = startWeekday;
    }

    public String getStyle() {
        return style;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        return styleClass;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    protected String buildConfig(FacesContext context) {
        Map<String, Object> attrs = getAttributes();
        StringBuilder config = new StringBuilder("{");

        ResourceBundle bundle = ScalesUtil.getBundle("com.sun.mojarra.scales.MessageResources", (String)attrs.get("locale"), this.getClass());


        Integer[] fieldPositions = calculateFieldPositions(); // {month, day, year} icky, I know, but it will do for now

        ScalesUtil.appendToConfig(config, "clientId", getClientId(context), true, false);
        ScalesUtil.appendToConfig(config, "startMonth", ScalesUtil.formatDate((Date) getValue(), "MM/yyyy", true), true, true);
        ScalesUtil.appendToConfig(config, "selectedDate", ScalesUtil.formatDate((Date) getValue(), "MM/dd/yyyy", false), true, true);
        ScalesUtil.appendToConfig(config, "showWeekdays", (Boolean)attrs.get("showWeekdays"), false, true);
        ScalesUtil.appendToConfig(config, "startWeekday", (Integer)attrs.get("startWeekday"), false, true);
        ScalesUtil.appendToConfig(config, "showWeekHeader", (Boolean)attrs.get("showWeekHeader"), false, true);
        ScalesUtil.appendToConfig(config, "showWeekFooter", (Boolean)attrs.get("showWeekFooter"), false, true);
        ScalesUtil.appendToConfig(config, "hideBlankWeeks", (Boolean)attrs.get("hideBlankWeeks"), false, true);
        ScalesUtil.appendToConfigIfNotNull(config, "minDate", (Date)attrs.get("minDate"), true, true);
        ScalesUtil.appendToConfigIfNotNull(config, "maxDate", (Date)attrs.get("maxDate"), true, true);
        ScalesUtil.appendToConfigIfNotNull(config, "onChange", (String)attrs.get("onChange"), true, true);
        ScalesUtil.appendToConfig(config, "navigator", configureNavigator(), false, true);
        ScalesUtil.appendToConfig(config, "popup", (Boolean)attrs.get("popup"), false, true);
        ScalesUtil.appendToConfig(config, "dayPosition", fieldPositions[1], false, true);
        ScalesUtil.appendToConfig(config, "monthPosition", fieldPositions[0], false, true);
        ScalesUtil.appendToConfig(config, "yearPosition", fieldPositions[2], false, true);
        ScalesUtil.appendToConfig(config, "format", (String)attrs.get("format"), true, true);
        ScalesUtil.appendToConfig(config, "dateFieldDelimiter", bundle.getString("date_field_delimiter"), true, true);
        ScalesUtil.appendToConfig(config, "mdyDayPosition", bundle.getString("mdy_day_position"), false, true);
        ScalesUtil.appendToConfig(config, "mdyMonthPosition", bundle.getString("mdy_month_position"), false, true);
        ScalesUtil.appendToConfig(config, "mdyYearPosition", bundle.getString("mdy_year_position"), false, true);
        ScalesUtil.appendToConfig(config, "mdDayPosition", bundle.getString("md_day_position"), false, true);
        ScalesUtil.appendToConfig(config, "mdMonthPosition", bundle.getString("md_month_position"), false, true);
        ScalesUtil.appendToConfig(config, "monthsShort", "[" + bundle.getString("months_short") + "]", false, true);
        ScalesUtil.appendToConfig(config, "monthsLong", "[" + bundle.getString("months_long") + "]", false, true);
        ScalesUtil.appendToConfig(config, "weekdays1Char", "[" + bundle.getString("weekdays_1char") + "]", false, true);
        ScalesUtil.appendToConfig(config, "weekdaysShort", "[" + bundle.getString("weekdays_short") + "]", false, true);
        ScalesUtil.appendToConfig(config, "weekdaysMedium", "[" + bundle.getString("weekdays_medium") + "]", false, true);
        ScalesUtil.appendToConfig(config, "weekdaysLong", "[" + bundle.getString("weekdays_long") + "]", false, true);
        return config.append("}").toString();
    }

    @Override
    public void decode(FacesContext facesContext) {
        Map requestMap = facesContext.getExternalContext().getRequestParameterMap();
        String clientId = getClientId(facesContext);

        try {
            Date date = null;
            String string_submit_val = ((String) requestMap.get(clientId));
            if (string_submit_val != null) {
                DateFormat df = new SimpleDateFormat((String)getAttributes().get("format"));
                date = df.parse(string_submit_val);
            }

            setSubmittedValue(date);
        } catch (ParseException ex) {
            facesContext.addMessage(this.getClientId(facesContext),
                                    new FacesMessage(FacesMessage.SEVERITY_ERROR, ex.getMessage(), null));
            Logger.getLogger(DateSelector.class.getName()).severe(ex.getMessage());
        }
    }

    private Integer[] calculateFieldPositions() {
        Integer[] positions = new Integer[]{1, 2, 3};
        String localFormat = ((String)getAttributes().get("format")).toLowerCase();
        String[] parts = localFormat.split("/");
        if (parts.length == 1) {
            parts = localFormat.split("-");
        }

        int index = 1;
        // Set values based on format provided
        for (int i = 0; i < parts.length; i++) {
            if (parts[i].startsWith("m")) {
                index = 0;
            } else if (parts[i].startsWith("y")) {
                index = 2;
            }
            positions[index] = i + 1;
        }

        return positions;
    }

    private String configureNavigator() {
        String retvalue = (String)getAttributes().get("navigator");

        ResourceBundle bundle = ScalesUtil.getBundle("com.sun.mojarra.scales.MessageResources", (String)getAttributes().get("locale"), this.getClass());
        if (!"true".equalsIgnoreCase(retvalue)) {
            retvalue = "{strings: {" +
                    "month:\"" + bundle.getString("navigator_month") + "\"," +
                    "year:\"" + bundle.getString("navigator_year") + "\"," +
                    "submit: \"" + bundle.getString("navigator_submit") + "\"," +
                    "cancel: \"" + bundle.getString("navigator_cancel") + "\"," +
                    "invalidYear: \"" + bundle.getString("navigator_invalid_year") + "\"}," +
                    "monthFormat: " + bundle.getString("navigator_month_format") + "," +
                    "initialFocus: \"" + bundle.getString("navigator_initial_focus") + "\"}";
        }

        return retvalue;
    }

    @Override
    public void restoreState(FacesContext _context, Object _state) {
        this._state = (Object[]) _state;
        super.restoreState(_context, this._state[0]);
        
        hideBlankWeeks = (Boolean) this._state[1];
        locale = (String) this._state[2];
        multiSelect = (Boolean) this._state[3];
        navigator = (String) this._state[4];
        onChange = (String) this._state[5];
        popup = (Boolean) this._state[6];
        showWeekdays = (Boolean) this._state[7];
        showWeekFooter = (Boolean) this._state[8];
        showWeekHeader = (Boolean) this._state[9];
        startWeekday = (Integer) this._state[10];
        minDate = (Date) this._state[11];
        maxDate = (Date) this._state[12];
        showMenus = (Boolean) this._state[13];
        format = (String) this._state[14];
        enableManualInput = (Boolean) this._state[15];
        style = (String) this._state[16];
        styleClass = (String) this._state[17];
    }

    @Override
    public Object saveState(FacesContext _context) {
        if (_state == null) {
            _state = new Object[18];
        }
        _state[0] = super.saveState(_context);
        _state[1] = hideBlankWeeks;
        _state[2] = locale;
        _state[3] = multiSelect;
        _state[4] = navigator;
        _state[5] = onChange;
        _state[6] = popup;
        _state[7] = showWeekdays;
        _state[8] = showWeekFooter;
        _state[9] = showWeekHeader;
        _state[10] = startWeekday;
        _state[11] = minDate;
        _state[12] = maxDate;
        _state[13] = showMenus;
        _state[14] = format;
        _state[15] = enableManualInput;
        _state[16] = style;
        _state[17] = styleClass;

        return _state;
    }
}