/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
/**
 * 
 */
package com.sun.mojarra.scales.component;

import com.sun.mojarra.scales.model.FileHolder;
import com.sun.mojarra.scales.util.RenderingHelper;
import com.sun.mojarra.scales.util.ScalesUtil;

import java.io.IOException;
import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * @author Jason Lee
 *
 */
public class MultiFileUpload extends UIOutput {

    public static final String COMPONENT_FAMILY = "com.sun.mojarra.scales.MultiFileUpload";
    public static final String COMPONENT_TYPE = COMPONENT_FAMILY;
    public static final String RENDERER_TYPE = COMPONENT_FAMILY;
    public static final String[] DEP_JARS = {
        "/mojarra-scales-upload-applet-1.3.1.jar",
        "/commons-fileupload-1.1.1.jar",
        "/commons-io-1.2.jar",
        "/commons-codec-1.3.jar",
        "/commons-logging-1.0.4.jar",
        "/commons-httpclient-3.1.jar"
    };
    public static final String REQUEST_PARAM = "componentId";
    public static final String UPLOAD_URI = "/mojarra___MultiFileUpload__fileupload";
    private Object[] _state = null;
    protected String buttonText;
    protected String destinationUrl;
    protected String fileFilter;
    protected FileHolder fileHolder;
    protected String height;
    protected Integer maxFileSize;
    protected String startDir;
    protected String width;

    public MultiFileUpload() {
        setRendererType(RENDERER_TYPE);
    }

    @Override
    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    @Override
    public void encodeBegin(FacesContext context) throws IOException {
        super.encodeBegin(context);

        // suppress rendering if "rendered" property on the component is false.
        if (!isRendered()) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        String clientId = getClientId(context);

        String sessionId = "";
        Object session = context.getExternalContext().getSession(true);
        if (session != null) {
            if (session instanceof HttpSession) {
                sessionId = ((HttpSession) session).getId();
            }
        }

        writer.startElement("applet", this);
        writer.writeAttribute("id", getClientId(context), "id");
        writer.writeAttribute("code", "com.sun.mojarra.scales.web.applet.upload.FullApplet", "code");
        writer.writeAttribute("name", "ScalesMultiFileUpload", "name");
        writer.writeAttribute("archive", getDepJars(context), "archive");
        writer.writeAttribute("width", width, "width");
        writer.writeAttribute("height", height, "height");
        RenderingHelper.outputElement(writer, this, "param", new String[][]{{"name", "type"}, {"value", "application/x-java-applet;version=1.5"}});
        RenderingHelper.outputElement(writer, this, "param", new String[][]{{"name", "uploadUrl"},
                                                                            {"value", context.getApplication().getViewHandler().getActionURL(context, generateUri(context, UPLOAD_URI)) + "?" + REQUEST_PARAM + "=" + clientId}});
        RenderingHelper.outputElement(writer, this, "param", new String[][]{{"name", "startdir"}, {"value", getStartDir()}});
        RenderingHelper.outputElement(writer, this, "param", new String[][]{{"name", "buttonText"}, {"value", getButtonText()}});
        RenderingHelper.outputElement(writer, this, "param", new String[][]{{"name", "fileFilter"}, {"value", getFileFilter()}});
        RenderingHelper.outputElement(writer, this, "param", new String[][]{{"name", "sessionId"}, {"value", sessionId}});
        RenderingHelper.outputElement(writer, this, "param", new String[][]{{"name", "maxFileSize"}, {"value", getMaxFileSize().toString()}});
        writer.write("Java 5 or higher plugin required.");
        writer.endElement("applet");
        ((HttpServletRequest)context.getExternalContext().getRequest()).getSession().setAttribute("HtmlMultiFileUpload-" + clientId, this);
    }

    public String getButtonText() {
        return ScalesUtil.getPropertyValue(this, buttonText, "buttonText", "Upload Files");
    }

    public void setButtonText(String buttonText) {
        this.buttonText = buttonText;
    }

    public String getDestinationUrl() {
        return ScalesUtil.getPropertyValue(this, destinationUrl, "destinationUrl", null);
    }

    public void setDestinationUrl(String destinationUrl) {
        this.destinationUrl = destinationUrl;
    }

    public String getFileFilter() {
        return ScalesUtil.getPropertyValue(this, fileFilter, "fileFilter", "");
    }

    public void setFileFilter(String fileFilter) {
        this.fileFilter = fileFilter;
    }

    public FileHolder getFileHolder() {
        return ScalesUtil.getPropertyValue(this, fileHolder, "fileHolder", null);
    }

    public void setFileHolder(FileHolder fileHolder) {
        this.fileHolder = fileHolder;
    }

    public String getHeight() {
        return ScalesUtil.getPropertyValue(this, height, "height", "250px");
    }

    public void setHeight(String height) {
        this.height = height;
    }

    public Integer getMaxFileSize() {
        return ScalesUtil.getPropertyValue(this, maxFileSize, "maxFileSize", 0);
    }

    public void setMaxFileSize(Integer maxFileSize) {
        this.maxFileSize = maxFileSize;
    }

    public String getStartDir() {
        return ScalesUtil.getPropertyValue(this, startDir, "startDir", "/");
    }

    public void setStartDir(String startDir) {
        this.startDir = startDir;
    }

    public String getWidth() {
        return ScalesUtil.getPropertyValue(this, width, "width", "500px");
    }

    public void setWidth(String width) {
        this.width = width;
    }

    @Override
    public void restoreState(FacesContext _context, Object _state) {
        this._state = (Object[]) _state;
        super.restoreState(_context, this._state[0]);
        fileHolder = (FileHolder) this._state[1];
        destinationUrl = (String) this._state[2];
        height = (String) this._state[3];
        maxFileSize = (Integer) this._state[4];
        startDir = (String) this._state[5];
        width = (String) this._state[6];
        buttonText = (String) this._state[7];
        fileFilter = (String) this._state[8];
    }

    @Override
    public Object saveState(FacesContext _context) {
        if (_state == null) {
            _state = new Object[9];
        }
        _state[0] = super.saveState(_context);
        _state[1] = fileHolder;
        _state[2] = destinationUrl;
        _state[3] = height;
        _state[4] = maxFileSize;
        _state[5] = startDir = "/";
        _state[6] = width;
        _state[7] = buttonText;
        _state[8] = fileFilter;

        return _state;
    }

    // TODO:  Look at the ScalesUtil method by the same name.  There's a good opportunity
    // for a refactor here.  Thank you, merging code bases. :)
    protected String generateUri(FacesContext context, String target) {
        String uri = "";
        String mapping = ScalesUtil.getFacesMapping(context);
        if (ScalesUtil.isPrefixMapped(mapping)) {
            uri = "/" + target + "." + mapping;
        } else {
            uri = target + mapping;
        }

        return uri;
    }

    protected String getDepJars(FacesContext context) {
        StringBuilder deps = new StringBuilder();
        String sep = "";

        for (String dep : DEP_JARS) {
            String depURI = context.getExternalContext().getRequestContextPath() +
                    ScalesUtil.createResourceUrl(context, dep);
            deps.append(sep).append(depURI);
            sep = ",";
        }

        return deps.toString();
    }
}
