package com.sun.mojarra.scales.component;

import com.sun.mojarra.scales.util.CssUtil;
import com.sun.mojarra.scales.util.RenderingHelper;
import com.sun.mojarra.scales.util.ScalesUtil;
import com.sun.mojarra.scales.util.YuiConstants;
import java.io.IOException;
import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 *
 * @author Jason Lee
 */
public class Panel extends UIOutput {

    public static final String COMPONENT_FAMILY = "com.sun.mojarra.scales.Panel";
    public static final String RENDERER_TYPE = COMPONENT_FAMILY;    // Component Properties
    private Object[] _state = null;
    protected String header;
    protected String body;
    protected String footer;
    protected String width;
    protected String height;
    protected Boolean fixedCenter;
    protected Boolean constrainToViewPort;
    protected Boolean visible;
    protected Boolean closable;
    protected Boolean draggable;
    protected String underlay;
    protected Boolean modal;
    protected Integer left;
    protected Integer top;
    protected Boolean flat;
    protected String style;
    protected String styleClass;

    public Panel() {
        super();
        setRendererType(RENDERER_TYPE);

        FacesContext context = FacesContext.getCurrentInstance();
        if (context != null) {
            String baseUrl = context.getExternalContext().getRequestContextPath();
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_CONTAINER));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_SCALES));
            CssUtil.linkContainerOverrides(context);
        }
    }

    @Override
    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    @Override
    public void encodeBegin(FacesContext context) throws IOException {
        super.encodeBegin(context);

        // suppress rendering if "rendered" property on the component is false.
        if (!isRendered()) {
            return;
        }
        String clientId = getClientId(context);

        ResponseWriter writer = context.getResponseWriter();
        writer.startElement("div", this);
        writer.writeAttribute("id", clientId, "id");
        if (styleClass != null) {
            writer.writeAttribute("class", getStyleClass(), "class");
        }
        writer.writeAttribute("style", "visibility: hidden; " + ((getStyle() != null) ? getStyle() : ""), "style");
        if (header != null) {
            writer.startElement("div", this);
            writer.writeAttribute("id", clientId + "_header", "id");
            writer.writeAttribute("class", "hd", "class");
            writer.write(header);
            writer.endElement("div");
        }
        writer.startElement("div", this);
        writer.writeAttribute("id", getClientId(context) + "_body", "id");
        writer.writeAttribute("class", "bd", "class");

    }

    @Override
    public void encodeEnd(FacesContext context) throws IOException {
        super.encodeEnd(context);
        
        if (!isRendered()) {
            return;
        }
        String clientId = getClientId(context);

        ResponseWriter writer = context.getResponseWriter();
        writer.endElement("div");
        if (footer != null) {
            writer.startElement("div", this);
            writer.writeAttribute("id", clientId + "_footer", "id");
            writer.writeAttribute("class", "ft", "class");
            writer.write(footer);
            writer.endElement("div");
        }
        writer.endElement("div");
        RenderingHelper.writeOnContentReady(writer, clientId, "scales.createPanel('" + clientId + "', {" + buildConfig() + "});");
    }

    protected String buildConfig() {
        StringBuilder builder = new StringBuilder();

        ScalesUtil.appendToConfig(builder, "width", getWidth(), true, false);
        ScalesUtil.appendToConfig(builder, "height", getHeight(), true, true);
        ScalesUtil.appendToConfigIfNotNull(builder, "x", getLeft(), false, true);
        ScalesUtil.appendToConfigIfNotNull(builder, "y", getTop(), false, true);
        ScalesUtil.appendToConfig(builder, "fixedcenter", getFixedCenter(), false, true);
        ScalesUtil.appendToConfig(builder, "constraintoviewport", getConstrainToViewPort(), false, true);
        ScalesUtil.appendToConfig(builder, "underlay", getUnderlay(), true, true);
        ScalesUtil.appendToConfig(builder, "close", getClosable(), false, true);
        ScalesUtil.appendToConfig(builder, "visible", getVisible(), false, true);
        ScalesUtil.appendToConfig(builder, "draggable", getDraggable(), false, true);
        ScalesUtil.appendToConfig(builder, "modal", getModal(), false, true);
        ScalesUtil.appendToConfig(builder, "flat", getFlat(), false, true);

        return builder.toString();
    }

    public String getBody() {
        return ScalesUtil.getPropertyValue(this, body, "body", null);
    }

    public void setBody(String body) {
        this.body = body;
    }

    public Boolean getClosable() {
        return ScalesUtil.getPropertyValue(this, closable, "closable", Boolean.FALSE);
    }

    public void setClosable(Boolean closable) {
        this.closable = closable;
    }

    public Boolean getConstrainToViewPort() {
        return ScalesUtil.getPropertyValue(this, constrainToViewPort, "constrainToViewPort", Boolean.TRUE);
    }

    public void setConstrainToViewPort(Boolean constrainToViewPort) {
        this.constrainToViewPort = constrainToViewPort;
    }

    public Boolean getDraggable() {
        return ScalesUtil.getPropertyValue(this, draggable, "draggable", Boolean.TRUE);
    }

    public void setDraggable(Boolean draggable) {
        this.draggable = draggable;
        if (draggable) {
            setFlat(false);
        }
    }

    public Boolean getFixedCenter() {
        return ScalesUtil.getPropertyValue(this, fixedCenter, "fixedCenter", Boolean.FALSE);
    }

    public void setFixedCenter(Boolean fixedCenter) {
        this.fixedCenter = fixedCenter;
    }

    public Boolean getFlat() {
        return ScalesUtil.getPropertyValue(this, flat, "flat", Boolean.FALSE);
    }

    public void setFlat(Boolean flat) {
        this.flat = flat;
        if (flat) {
            setDraggable(false);
        }
    }

    public String getFooter() {
        return ScalesUtil.getPropertyValue(this, footer, "footer", null);
    }

    public void setFooter(String footer) {
        this.footer = footer;
    }

    public String getHeader() {
        return ScalesUtil.getPropertyValue(this, header, "header", null);
    }

    public void setHeader(String header) {
        this.header = header;
    }

    public String getHeight() {
        return ScalesUtil.getPropertyValue(this, height, "height", "auto");
    }

    public void setHeight(String height) {
        this.height = height;
    }

    public Integer getLeft() {
        return ScalesUtil.getPropertyValue(this, left, "left", null);
    }

    public void setLeft(Integer left) {
        this.left = left;
    }

    public Boolean getModal() {
        return ScalesUtil.getPropertyValue(this, modal, "modal", Boolean.FALSE);
    }

    public void setModal(Boolean modal) {
        this.modal = modal;
    }

    public String getStyle() {
        return ScalesUtil.getPropertyValue(this, style, "style", "");
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        return ScalesUtil.getPropertyValue(this, styleClass, "styleClass", "");
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    public Integer getTop() {
        return ScalesUtil.getPropertyValue(this, top, "top", null);
    }

    public void setTop(Integer top) {
        this.top = top;
    }

    public String getUnderlay() {
        return ScalesUtil.getPropertyValue(this, underlay, "underlay", "shadow");
    }

    public void setUnderlay(String underlay) {
        this.underlay = underlay;
    }

    public Boolean getVisible() {
        return ScalesUtil.getPropertyValue(this, visible, "visible", Boolean.TRUE);
    }

    public void setVisible(Boolean visible) {
        this.visible = visible;
    }

    public String getWidth() {
        return ScalesUtil.getPropertyValue(this, width, "width", "100%");
    }

    public void setWidth(String width) {
        this.width = width;
    }

    @Override
    public Object saveState(FacesContext context) {
        if (_state == null) {
            _state = new Object[18];
        }
        _state[0] = super.saveState(context);
        _state[1] = header;
        _state[2] = body;
        _state[3] = footer;
        _state[4] = width;
        _state[5] = height;
        _state[6] = fixedCenter;
        _state[7] = constrainToViewPort;
        _state[8] = visible;
        _state[9] = closable;
        _state[10] = draggable;
        _state[11] = underlay;
        _state[12] = modal;
        _state[13] = left;
        _state[14] = top;
        _state[15] = flat;
        _state[16] = style;
        _state[17] = styleClass;
        return _state;
    }

    @Override
    public void restoreState(FacesContext context, Object state) {
        this._state = (Object[]) state;
        super.restoreState(context, this._state[0]);
        header = (String) this._state[1];
        body = (String) this._state[2];
        footer = (String) this._state[3];
        width = (String) this._state[4];
        height = (String) this._state[5];
        fixedCenter = (Boolean) this._state[6];
        constrainToViewPort = (Boolean) this._state[7];
        visible = (Boolean) this._state[8];
        closable = (Boolean) this._state[9];
        draggable = (Boolean) this._state[10];
        underlay = (String) this._state[11];
        modal = (Boolean) this._state[12];
        left = (Integer) this._state[13];
        top = (Integer) this._state[14];
        flat = (Boolean) this._state[15];
        style = (String) this._state[16];
        styleClass = (String) this._state[17];
    }
}