/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package com.sun.mojarra.scales.component;

import com.sun.mojarra.scales.util.CssUtil;
import com.sun.mojarra.scales.util.RenderingHelper;
import com.sun.mojarra.scales.util.ScalesUtil;
import com.sun.mojarra.scales.util.YuiConstants;
import java.io.IOException;
import java.util.Map;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * @author Jason Lee
 */
public class YuiSlider extends UIInput {

    public static final String COMPONENT_FAMILY = "com.sun.mojarra.scales.YuiSlider";
    public static final String RENDERER_TYPE = COMPONENT_FAMILY;
    public static final String COMPONENT_TYPE = COMPONENT_FAMILY;
    private Boolean animate;
    private Double animationDuration;
    private Integer min;
    private Float scaleFactor;
    private Integer max;
    private Integer tick;
    private String orientation;
    private String update;
    private String style;
    private String styleClass;
    private Object[] _state = null;

    public YuiSlider() {
        super();
        setRendererType(RENDERER_TYPE);

        FacesContext context = FacesContext.getCurrentInstance();
        if (context != null) {
            String baseUrl = context.getExternalContext().getRequestContextPath();
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_SLIDER));
            Links.addCss(Links.EXTERNAL_STYLESHEETS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.CSS_SKIN_SAM));
            CssUtil.linkSliderOverrides(context);
        }
    }

    @Override
    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    @Override
    public void encodeBegin(FacesContext context) throws IOException {
        super.encodeBegin(context);

        // suppress rendering if "rendered" property on the component is false.
        if (!isRendered()) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        String clientId = getClientId(context);

        boolean isHorizontal = "horizontal".equalsIgnoreCase(getOrientation());
        String baseUrl = ScalesUtil.getStaticResourceUrl(context, "");
        Integer value = (Integer) getValue();
        final int counterDimension = Math.round((getMax() - getMin()) * getScaleFactor() + 14);

        RenderingHelper.outputElement(writer, this, "input",
                                      new String[][]{{"type", "hidden"}, {"id", clientId}, {"name", clientId}, {"value", (value != null) ? value.toString() : "0"}});
        writer.startElement("div", this);
        writer.writeAttribute("id", clientId + "_slider", "id");
        writer.writeAttribute("class", getStyleClass(), "class");
        writer.writeAttribute("style", "background-image: url('" + baseUrl + "scales/img/slider-bg-" + getOrientation() + ".gif'); background-repeat: repeat-" +
                (isHorizontal ? "x" : "y") + ";" + (isHorizontal ? "height" : "width") + ": 28px;" +
                (isHorizontal ? "width" : "height") + ": " + counterDimension + "px; " + getStyle(), "style");

        writer.startElement("div", this);
        writer.writeAttribute("id", clientId + "_sliderthumb", "id");
        writer.startElement("img", this);
        writer.writeAttribute("src", baseUrl + "scales/img/slider-thumb-" + getOrientation().toLowerCase() + ".gif", "src");
        writer.endElement("img");
        writer.endElement("div");
        writer.endElement("div");
        RenderingHelper.writeOnDomReady(writer, "scales.createSlider('" + clientId + "', {" + buildConfig() + "});");
    }

    protected String buildConfig() {
        StringBuilder builder = new StringBuilder();

        ScalesUtil.appendToConfig(builder, "orientation", getOrientation(), true, false);
        ScalesUtil.appendToConfig(builder, "min", getMin(), false, true);
        ScalesUtil.appendToConfig(builder, "max", getMax(), false, true);
        ScalesUtil.appendToConfig(builder, "tick", getTick(), false, true);
        ScalesUtil.appendToConfig(builder, "scaleFactor", getScaleFactor(), false, true);
        ScalesUtil.appendToConfig(builder, "animate", getAnimate(), false, true);
        ScalesUtil.appendToConfig(builder, "animationDuration", getAnimationDuration(), false, true);
        ScalesUtil.appendToConfigIfNotNull(builder, "update", getUpdate(), true, true);
        ScalesUtil.appendToConfig(builder, "value", getValue(), false, true);

        return builder.toString();
    }

    @Override
    public void decode(FacesContext facesContext) {
        Map requestMap = facesContext.getExternalContext().getRequestParameterMap();
        String clientId = getClientId(facesContext);

        String string_submit_val = ((String) requestMap.get(clientId));
        Integer integer = Integer.decode(string_submit_val);

        setSubmittedValue(integer);
    }

    public Boolean getAnimate() {
        return ScalesUtil.getPropertyValue(this, animate, "animate", Boolean.TRUE);
    }

    public void setAnimate(Boolean animate) {
        this.animate = animate;
    }

    public Double getAnimationDuration() {
        return ScalesUtil.getPropertyValue(this, animationDuration, "animationDuration", 0.2d);
    }

    public void setAnimationDuration(Double animationDuration) {
        this.animationDuration = animationDuration;
    }

    public String getUpdate() {
        return ScalesUtil.getPropertyValue(this, update, "update", null);
    }

    public void setUpdate(String update) {
        this.update = update;
    }

    public int getMax() {
        return ScalesUtil.getPropertyValue(this, max, "max", 100);
    }

    public void setMax(int max) {
        this.max = max;
    }

    public int getMin() {
        return ScalesUtil.getPropertyValue(this, min, "min", 1);
    }

    public void setMin(int min) {
        this.min = min;
    }

    public String getOrientation() {
        return ScalesUtil.getPropertyValue(this, orientation, "orientation", "horoizontal");
    }

    public void setOrientation(String orientation) {
        this.orientation = orientation;
    }

    public float getScaleFactor() {
        return ScalesUtil.getPropertyValue(this, scaleFactor, "scaleFactor", 1.0f);
    }

    public void setScaleFactor(float scaleFactor) {
        this.scaleFactor = scaleFactor;
    }

    public String getStyle() {
        return ScalesUtil.getPropertyValue(this, style, "style", "");
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        return ScalesUtil.getPropertyValue(this, styleClass, "styleClass", "");
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    public int getTick() {
        return ScalesUtil.getPropertyValue(this, tick, "tick", 1);
    }

    public void setTick(int tick) {
        this.tick = tick;
    }

    @Override
    public void restoreState(FacesContext _context, Object _state) {
        this._state = (Object[]) _state;
        super.restoreState(_context, this._state[0]);
        animate = (Boolean) this._state[1];
        animationDuration = (Double) this._state[2];
        min = (Integer) this._state[4];
        style = (String) this._state[5];
        styleClass = (String) this._state[6];
        scaleFactor = (Float) this._state[7];
        max = (Integer) this._state[8];
        tick = (Integer) this._state[9];
        orientation = (String) this._state[10];
        update = (String) this._state[11];
    }

    @Override
    public Object saveState(FacesContext _context) {
        if (_state == null) {
            _state = new Object[12];
        }
        _state[0] = super.saveState(_context);
        _state[1] = animate;
        _state[2] = animationDuration;
        _state[4] = min;
        _state[5] = style;
        _state[6] = styleClass;
        _state[7] = scaleFactor;
        _state[8] = max;
        _state[9] = tick;
        _state[10] = orientation;
        _state[11] = update;
        return _state;
    }
}