/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
/**
 * 
 */
package com.sun.mojarra.scales.component;

import com.sun.mojarra.scales.util.CssUtil;
import com.sun.mojarra.scales.util.RenderingHelper;
import com.sun.mojarra.scales.util.ScalesUtil;
import com.sun.mojarra.scales.util.YuiConstants;
import java.io.IOException;
import javax.faces.component.UIComponent;
import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * @author Jason Lee
 *
 */
public class YuiTabView extends UIOutput {

    public static final String COMPONENT_FAMILY = "com.sun.mojarra.scales.YuiTabView";
    public static final String COMPONENT_TYPE = COMPONENT_FAMILY;
    public static final String RENDERER_TYPE = COMPONENT_FAMILY;
    protected String minHeight;
    protected String maxHeight; // "dynamic", "auto", or "##px"
    protected String orientation; // top, right, bottom, or left
    protected String style;
    protected String styleClass;
    private Object[] _state;

    public YuiTabView() {
        setRendererType(RENDERER_TYPE);

        FacesContext context = FacesContext.getCurrentInstance();
        if (context != null) {
            String baseUrl = context.getExternalContext().getRequestContextPath();
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_TABVIEW));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_CONTAINER));
            Links.addScript(Links.EXTERNAL_SCRIPTS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.JS_SCALES));
            Links.addCss(Links.EXTERNAL_STYLESHEETS, baseUrl + ScalesUtil.createResourceUrl(context, YuiConstants.CSS_SKIN_SAM));
            CssUtil.linkTabViewOverrides(context);
            CssUtil.linkContainerOverrides(context);
            CssUtil.linkButtonOverrides(context);
        }
    }

    @Override
    public String getFamily() {
        return COMPONENT_FAMILY;
    }

    @Override
    public void encodeBegin(FacesContext context) throws IOException {
        super.encodeBegin(context);

        // suppress rendering if "rendered" property on the component is false.
        if (!isRendered()) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        String clientId = getClientId(context);

        writer.startElement("div", this);
        writer.writeAttribute("id", clientId, "id");
        writer.writeAttribute("class", "yui-navset " + getStyleClass(), "class");
        writer.writeAttribute("style", getStyle(), "style");
        writer.startElement("ul", this);
        writer.writeAttribute("class", "yui-nav", "class");
        for (UIComponent child : getChildren()) {
            if (child instanceof YuiTab) {
                renderTabLabel(writer, context, (YuiTab) child);
            }
        }
        writer.endElement("ul");
        writer.startElement("div", this);
        writer.writeAttribute("class", "yui-content", "class");
    }

    @Override
    public void encodeEnd(FacesContext context) throws IOException {
        super.encodeEnd(context);
        
        if (!isRendered()) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        writer.endElement("div");
        writer.endElement("div");
        RenderingHelper.writeOnDomReady(writer, "scales.createTabView('" + getClientId(context) + "', '" + getMaxHeight() + "');");
    }

    protected void renderTabLabel(ResponseWriter writer, FacesContext context, YuiTab tab) throws IOException {
        writer.startElement("li", tab);
        if (Boolean.TRUE.equals(tab.getActive())) {
            writer.writeAttribute("class", "selected", "class");
        } else if (Boolean.TRUE.equals(tab.getDisabled())) {
            writer.writeAttribute("class", "disabled", "class");
        }
        writer.write("<a><em>");
        UIComponent facet = tab.getFacet("label");
        if (facet != null) {
            facet.encodeAll(context);
        } else {
            writer.write(tab.getLabel());
        }
        writer.write("</em></a>");
        writer.endElement("li");
    }

    public String getMaxHeight() {
        return ScalesUtil.getPropertyValue(this, maxHeight, "maxHeight", "auto");
    }

    public void setMaxHeight(String maxHeight) {
        this.maxHeight = maxHeight;
    }

    public String getMinHeight() {
        return ScalesUtil.getPropertyValue(this, minHeight, "minHeight", null);
    }

    public void setMinHeight(String minHeight) {
        this.minHeight = minHeight;
    }

    public String getOrientation() {
        return ScalesUtil.getPropertyValue(this, orientation, "orientation", "top");
    }

    public void setOrientation(String orientation) {
        this.orientation = orientation;
    }

    public String getStyle() {
        return ScalesUtil.getPropertyValue(this, style, "style", "");
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        return ScalesUtil.getPropertyValue(this, styleClass, "styleClass", "");
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    @Override
    public void restoreState(FacesContext _context, Object _state) {
        this._state = (Object[]) _state;
        super.restoreState(_context, this._state[0]);
        minHeight = (String) this._state[1];
        maxHeight = (String) this._state[2];
        orientation = (String) this._state[3];
        style = (String) this._state[4];
        styleClass = (String) this._state[5];
    }

    @Override
    public Object saveState(FacesContext _context) {
        if (_state == null) {
            _state = new Object[6];
        }
        _state[0] = super.saveState(_context);
        _state[1] = minHeight;
        _state[2] = maxHeight;
        _state[3] = orientation;
        _state[4] = style;
        _state[5] = styleClass;
        return _state;
    }
}