/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

/**
 * 
 */
package com.sun.mojarra.scales.renderer;

import java.io.IOException;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.sun.mojarra.scales.component.YuiTreeNode;

/**
 * @author Jason Lee
 *
 */
//@Renderer(@Renderer.Renders(componentFamily="com.sun.mojarra.scales.YuiTreeNode"))
public class YuiTreeNodeRenderer extends AbstractRenderer {

    @Override
    public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
        if (context == null) {
            throw new NullPointerException("param 'context' is null");
        }
        if (component == null) {
            throw new NullPointerException("param 'component' is null");
        }
        if (!(component instanceof YuiTreeNode)) {
            throw new FacesException("Expected an instance of YuiTreeNode.  Found " +
                    component.getClass().getName() + ".");
        }

        // suppress rendering if "rendered" property on the component is false.
        if (!component.isRendered()) {
            return;
        }

        YuiTreeNode treeNode = (YuiTreeNode)component;
        ResponseWriter writer = context.getResponseWriter();

        UIComponent labelFacet = treeNode.getFacet("label");
        String label = null;
        if (labelFacet == null) {
            label = treeNode.getLabel();
            if (label == null) {
                throw new FacesException ("treeNode id " +
                    treeNode.getClientId(context) + " is missing both its label facet and the label attribute.  One must be specified (with the facet having the higher priority).");
            }
        }

        // Use .write() instead of the higher level ResponseWriters because of that
        // silly yuiConfig element and JSON. YUCK!  Until someone complains, though...
        writer.write("<li id=\"" + treeNode.getClientId(context) + "\"");
        if (treeNode.getExpanded()) {
            writer.write(" class=\"" + "expanded" + "\"");
        }
        writer.write(" yuiConfig='{\"jsfId\":\"" + treeNode.getClientId(context) +"\"}'>");
        if (labelFacet != null) {
            labelFacet.encodeAll(context);
        } else {
            writer.write(label);
        }
        if (treeNode.getChildCount() > 0) {
            writer.startElement("ul", null);
        }
    }

    @Override
    public void encodeChildren(FacesContext context, UIComponent component) throws IOException {
        if (!component.isRendered()) {
            return;
        }

        YuiTreeNode treeNode = (YuiTreeNode)component;
        ResponseWriter writer = context.getResponseWriter();
        for (UIComponent child : treeNode.getChildren()) {
            boolean isAnotherNode = child instanceof YuiTreeNode;
            if (!isAnotherNode) {
                writer.startElement("li", treeNode);
            }
            child.encodeAll(context);
            if (!isAnotherNode) {
                writer.endElement("li");
            }
        }
    }

    @Override
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
        YuiTreeNode treeNode = (YuiTreeNode)component;
        ResponseWriter writer = context.getResponseWriter();
        if (treeNode.getChildCount() > 0) {
            writer.endElement("ul");
        }
        writer.endElement("li");
        writer.write("\n");
    }

    @Override
    public boolean getRendersChildren() {
        return true;
    }
}