/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
/**
 * 
 */
package com.sun.mojarra.scales.renderer;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.sun.mojarra.scales.component.YuiTree;
import com.sun.mojarra.scales.component.YuiTreeNode;
import com.sun.mojarra.scales.model.ITreeNode;
import com.sun.mojarra.scales.util.RenderingHelper;
import com.sun.mojarra.scales.util.ScalesUtil;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.el.ValueExpression;

/**
 * @author Jason Lee
 *
 */
//@Renderer(@Renderer.Renders(componentFamily = "com.sun.mojarra.scales.YuiTree"))
public class YuiTreeRenderer extends AbstractRenderer {
    protected Map<String, YuiTreeNode> nodeTypes = new HashMap<String, YuiTreeNode>();

    @Override
    public void encodeBegin(FacesContext context, UIComponent component)
            throws IOException {

        if (context == null) {
            throw new NullPointerException("param 'context' is null");
        }
        if (component == null) {
            throw new NullPointerException("param 'component' is null");
        }

        // suppress rendering if "rendered" property on the component is false.
        if (!component.isRendered()) {
            return;
        }

        YuiTree tree = (YuiTree) component;
        ResponseWriter writer = context.getResponseWriter();

        writer.startElement("div", tree);
        writer.writeAttribute("id", component.getClientId(context), "id");
        ScalesUtil.renderPassThruAttributes(writer, component);

        writer.startElement("ul", null);

        // Build a map of our "templates"
        for (Map.Entry<String, UIComponent> facet : tree.getFacets().entrySet()) {
            YuiTreeNode node = (YuiTreeNode) context.getApplication().createComponent(YuiTreeNode.COMPONENT_TYPE);
//            node.getChildren().add(facet.getValue());
            node.getFacets().put("label", facet.getValue());
            nodeTypes.put(facet.getKey(), node);
        }
        @SuppressWarnings("unchecked")
        List<ITreeNode> model = (List<ITreeNode>) tree.getValue();
        if (model != null) {
            for (int i = 0; i < model.size(); i++) {
                ITreeNode item = model.get(i);
                encodeNode(context, writer, item, tree, i, tree.getVar());
            }
        } else { // Assuming a UIComponent-based approach
            processComponentSubTree(tree);
        }
    }


    @Override
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
        if (context == null) {
            throw new NullPointerException("param 'context' is null");
        }
        if (component == null) {
            throw new NullPointerException("param 'component' is null");
        }

        // suppress rendering if "rendered" property on the component is false.
        if (!component.isRendered()) {
            return;
        }

        YuiTree tree = (YuiTree) component;
        ResponseWriter writer = context.getResponseWriter();
        writer.endElement("ul");
        writer.endElement("div");

        RenderingHelper.writeOnDomReady(writer, "scales.createTree('" + tree.getClientId(context) + "', " + tree.getShowExpanded() +");");
    }

    @Override
    public boolean getRendersChildren() {
        return false;
    }

    protected void encodeNode(FacesContext context, ResponseWriter writer, ITreeNode modelItem, UIComponent parent,
                              int index, String varName) throws IOException {
        YuiTreeNode treeNode = nodeTypes.get(modelItem.getType());
        if (treeNode != null) {
            treeNode.setValue(modelItem);
//            treeNode.setId("node" + Integer.toString(index));
            ValueExpression ve = ScalesUtil.createValueExpression("#{" + varName + "}", String.class);
            Object backup = ve.getValue(context.getELContext());
            ve.setValue(context.getELContext(), modelItem);

            treeNode.encodeBegin(context);
            ve.setValue(context.getELContext(), backup);
            ve = null;
            // if there are children, prepare the markup, then iterate over the children
            List<ITreeNode> children = modelItem.getChildren();
            if (children != null) {
                writer.startElement("ul", treeNode);

                for (int i = 0; i < children.size(); i++) {
                    ITreeNode item = children.get(i);
                    encodeNode(context, writer, item, treeNode, index + i + 1, varName);
                }
                writer.endElement("ul");
            }
            treeNode.encodeEnd(context);
        }
    }

    // I don't like this approach, but I think it will have to be this way until we move to JSF 2
    private void processComponentSubTree(UIComponent start) {
        for (UIComponent child : start.getChildren()) {
            if (child instanceof YuiTreeNode) {
                YuiTreeNode node = (YuiTreeNode)child;
                if (Boolean.TRUE.equals(node.getExpanded())) {
                    applyExpandedUpTree(child);
                }
                processComponentSubTree(node);
            }
        }
    }

    private void applyExpandedUpTree(UIComponent node) {
        if (node instanceof YuiTreeNode) {
            ((YuiTreeNode)node).setExpanded(true);
        }
        UIComponent parent = node.getParent();
        if (!(parent instanceof YuiTree)) {
            applyExpandedUpTree(parent);
        }
    }
}